@@ -6,6 +6,9 @@ use crate::autocomplete::Autocomplete;
 use crate::history::History;
 use crate::*;
 
+pub type ShellResult<S> = Result<(), ShellError<S>>;
+pub type PollResult<'a, S> = Result<Option<Input<'a>>, ShellError<S>>;
+
 pub struct UShell<S, A, H, const COMMAND_LEN: usize> {
     serial: S,
     autocomplete: A,
@@ -56,7 +59,7 @@ where
         match self.serial.read() {
             Ok(byte) => {
                 if self.bypass {
-                    return Ok(Some(Input::raw(byte)));
+                    return Ok(Some(Input::Raw(byte)));
                 }
 
                 match byte {
@@ -79,7 +82,7 @@ where
                             RIGHT => self.dpad_right()?,
                             UP => self.dpad_up()?,
                             DOWN => self.dpad_down()?,
-                            _ => return Ok(Some(Input::control(control_byte))),
+                            _ => return Ok(Some(Input::Control(control_byte))),
                         }
                     }
                     _ if self.escape => {
@@ -91,18 +94,14 @@ where
                     control::CR => {
                         let cmd = from_utf8(&self.cmd_buf[..self.cmd_len])
                             .map_err(ShellError::BadInputError)?;
-                        self.history
-                            .push_top(cmd)
-                            .map_err(ShellError::HistoryError)?;
-
+                        self.history.push(cmd).map_err(ShellError::HistoryError)?;
                         self.cmd_len = 0;
                         self.cursor = 0;
-                        return Ok(Some(Input::command(byte, cmd)));
+                        return Ok(Some(Input::Command(byte, cmd)));
                     }
                     _ => self.write_at_cursor(byte)?,
                 };
-
-                Ok(Some(Input::raw(byte)))
+                Ok(Some(Input::Raw(byte)))
             }
             Err(nb::Error::WouldBlock) => Ok(None),
             Err(nb::Error::Other(err)) => Err(ShellError::ReadError(err)),
@@ -120,8 +119,14 @@ where
         block!(self.serial.write(control::BELL)).map_err(ShellError::WriteError)
     }
 
+    pub fn push_history(&mut self, cmd: &str) -> ShellResult<S> {
+        self.history.push(cmd).map_err(ShellError::HistoryError)
+    }
+
     fn write_at_cursor(&mut self, byte: u8) -> ShellResult<S> {
-        if self.cursor < self.cmd_len {
+        if self.cursor == self.cmd_buf.len() {
+            return self.bell();
+        } else if self.cursor < self.cmd_len {
             block!(self.serial.write(byte)).map_err(ShellError::WriteError)?;
 
             self.cmd_buf
@@ -185,7 +190,7 @@ where
     }
 
     fn dpad_up(&mut self) -> ShellResult<S> {
-        if self.cursor != 0 && self.cursor != self.cmd_len {
+        if self.cursor != self.cmd_len {
             return self.bell();
         }
         match self.history.go_back() {
@@ -195,7 +200,7 @@ where
     }
 
     fn dpad_down(&mut self) -> ShellResult<S> {
-        if self.cursor != 0 && self.cursor != self.cmd_len {
+        if self.cursor != self.cmd_len {
             return self.bell();
         }
         match self.history.go_forward() {
